import hashlib


class HashState:
    def __init__(self):
        self._state = hashlib.md5()

    def update(self, data: bytes):
        self._state.update(str(len(data)).encode("utf-8") + b":")
        self._state.update(data + b":")
        return self

    def record(self, *args):
        self._state.update(str(args).encode("utf-8"))
        return self

    def state(self):
        return self._state.hexdigest()



import base64
import os

from common.rpc.auth_utils import set_token_path
from common.rpc.ag_master import upload_zip, create_assignment


class Autograder:
    def __init__(self, course):
        self.course = course
        set_token_path(f"{os.path.expanduser('~')}/.sicp_token")

    def upload_zip(self, zip_file):
        """
        Uploads `filename` zip and returns the download url, overriding any
        existing filesOverrides existing
        """
        assert zip_file.endswith(".zip"), "Upload Error"

        with open(zip_file, "rb") as f:
            upload_zip(
                course=self.course,
                name=zip_file,
                file=base64.b64encode(f.read()).decode("ascii"),
            )

    def create_assignment(
        self, assign, script, zip_file, batch_size=100, grading_base="https://okpy.org"
    ):
        return create_assignment(
            name=assign,
            command=script,
            file=zip_file,
            batch_size=batch_size,
            grading_base=grading_base,
            course=self.course,
        )



from typing import List, Dict
from common.rpc.utils import (
    create_service,
    requires_master_secret,
    requires_access_token,
)

service = create_service(__name__, "ag-master")


@requires_master_secret
@service.route("/api/trigger_jobs")
def trigger_jobs(*, assignment_id: str, jobs: List[str]):
    ...


@service.route("/api/get_submission")
def get_submission(*, job_id: str) -> Dict:
    ...


@service.route("/api/handle_output")
def handle_output(*, output: str, job_id: str):
    ...


@service.route("/api/set_failure")
def set_failure(*, job_id: str, result: str):
    ...


@requires_access_token
@service.route("/api/upload_zip")
def upload_zip(*, course: str, name: str, file: str):
    ...


@requires_access_token
@service.route("/api/create_assignment")
def create_assignment(
    *,
    course: str,
    name: str,
    file: str,
    command: str,
    batch_size: int,
    grading_base: str,
) -> str:
    ...
